/*
 * U-Boot and JzRISC LCD controller definitions
 *
 * Copyright (C) 2001 Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Copyright (C) 2005-2007, Ingenic Semiconductor Inc.
 * Copyright (C) Xiangfu Liu <xiangfu@sharism.cc>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#ifndef __JZLCD_H__
#define __JZLCD_H__

#include <stdint.h>

void lcd_set_bpp(uint8_t bpp);
uint32_t lcd_ctrl_init();
void lcd_enable();
void lcd_disable();

/*
 * Framebuffer characteristics
 */
struct jzfb_info {
	uint32_t cfg;	/* panel mode and pin usage etc. */
	uint16_t w;
	uint16_t h;
	uint8_t bpp;	/* bit per pixel */
	uint32_t fclk;	/* frame clk */
	uint32_t hsw;	/* hsync width, in pclk */
	uint32_t vsw;	/* vsync width, in line count */
	uint32_t elw;	/* end of line, in pclk */
	uint32_t blw;	/* begin of line, in pclk */
	uint32_t efw;	/* end of frame, in line count */
	uint32_t bfw;	/* begin of frame, in line count */
};

/*
 * LCD controller stucture for JZSOC: JZ4740
 */
struct jz_fb_dma_descriptor {
	struct jz_fb_dma_descriptor *fdadr;	/* Frame descriptor address register */
	uint32_t fsadr;				/* Frame source address register */
	uint32_t fidr;				/* Frame ID register */
	uint32_t ldcmd;				/* Command register */
};

/*
 * Jz LCD info
 */
struct jz_mem_info {

	struct jz_fb_dma_descriptor *fdadr0;	/* physical address of frame/palette descriptor */
	struct jz_fb_dma_descriptor *fdadr1;	/* physical address of frame descriptor */

	/* DMA descriptors */
	struct jz_fb_dma_descriptor *dmadesc_fblow;
	struct jz_fb_dma_descriptor *dmadesc_fbhigh;
	struct jz_fb_dma_descriptor *dmadesc_palette;

	uint32_t screen;			/* address of frame buffer */
	uint32_t palette;			/* address of palette memory */
};

/*
 * Concise display characteristics with low-level structure reference
 */
typedef struct vidinfo {
	struct jz_mem_info jz_mem;
	struct jzfb_info *jz_fb;
} vidinfo_t;

/* Alignment/rounding macros. */

#define ALIGN(x,a)              __ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)    (((x)+(mask))&~(mask))

/* Transfer and display types. */

#define MODE_MASK		0x0f
#define MODE_TFT_GEN		0x00
#define MODE_TFT_SHARP		0x01
#define MODE_TFT_CASIO		0x02
#define MODE_TFT_SAMSUNG	0x03
#define MODE_CCIR656_NONINT	0x04
#define MODE_CCIR656_INT	0x05
#define MODE_STN_COLOR_SINGLE	0x08
#define MODE_STN_MONO_SINGLE	0x09
#define MODE_STN_COLOR_DUAL	0x0a
#define MODE_STN_MONO_DUAL	0x0b
#define MODE_8BIT_SERIAL_TFT    0x0c

#define MODE_TFT_18BIT          (1<<7)

#define STN_DAT_PIN1	(0x00 << 4)
#define STN_DAT_PIN2	(0x01 << 4)
#define STN_DAT_PIN4	(0x02 << 4)
#define STN_DAT_PIN8	(0x03 << 4)
#define STN_DAT_PINMASK	STN_DAT_PIN8

#define STFT_PSHI	(1 << 15)
#define STFT_CLSHI	(1 << 14)
#define STFT_SPLHI	(1 << 13)
#define STFT_REVHI	(1 << 12)

#define SYNC_MASTER	(0 << 16)
#define SYNC_SLAVE	(1 << 16)

#define DE_P		(0 << 9)
#define DE_N		(1 << 9)

#define PCLK_P		(0 << 10)
#define PCLK_N		(1 << 10)

#define HSYNC_P		(0 << 11)
#define HSYNC_N		(1 << 11)

#define VSYNC_P		(0 << 8)
#define VSYNC_N		(1 << 8)

#define DATA_NORMAL	(0 << 17)
#define DATA_INVERSE	(1 << 17)

#endif /* __JZLCD_H__ */
