#ifndef __TASK_GPIO_H__
#define __TASK_GPIO_H__

#include "memory.h"
#include "xburst_types.h"

/* Special task versions of GPIO operations. */

#define GPIO_BASE_PHYSICAL	(GPIO_BASE - KSEG1_BASE)

#define TASK_GPIO_PXPIN(n)	(TASK_GPIO_BASE + (0x00 + (n)*0x100)) /* PIN Level Register */

#define TASK_GPIO_PXDAT(n)	(TASK_GPIO_BASE + (0x10 + (n)*0x100)) /* Port Data Register */
#define TASK_GPIO_PXDATS(n)	(TASK_GPIO_BASE + (0x14 + (n)*0x100)) /* Port Data Set Register */
#define TASK_GPIO_PXDATC(n)	(TASK_GPIO_BASE + (0x18 + (n)*0x100)) /* Port Data Clear Register */

#define TASK_GPIO_PXIM(n)	(TASK_GPIO_BASE + (0x20 + (n)*0x100)) /* Interrupt Mask Register */
#define TASK_GPIO_PXIMS(n)	(TASK_GPIO_BASE + (0x24 + (n)*0x100)) /* Interrupt Mask Set Reg */
#define TASK_GPIO_PXIMC(n)	(TASK_GPIO_BASE + (0x28 + (n)*0x100)) /* Interrupt Mask Clear Reg */

#define TASK_GPIO_PXPE(n)	(TASK_GPIO_BASE + (0x30 + (n)*0x100)) /* Pull Enable Register */
#define TASK_GPIO_PXPES(n)	(TASK_GPIO_BASE + (0x34 + (n)*0x100)) /* Pull Enable Set Reg. */
#define TASK_GPIO_PXPEC(n)	(TASK_GPIO_BASE + (0x38 + (n)*0x100)) /* Pull Enable Clear Reg. */

#define TASK_GPIO_PXFUN(n)	(TASK_GPIO_BASE + (0x40 + (n)*0x100)) /* Function Register */
#define TASK_GPIO_PXFUNS(n)	(TASK_GPIO_BASE + (0x44 + (n)*0x100)) /* Function Set Register */
#define TASK_GPIO_PXFUNC(n)	(TASK_GPIO_BASE + (0x48 + (n)*0x100)) /* Function Clear Register */

#define TASK_GPIO_PXSEL(n)	(TASK_GPIO_BASE + (0x50 + (n)*0x100)) /* Select Register */
#define TASK_GPIO_PXSELS(n)	(TASK_GPIO_BASE + (0x54 + (n)*0x100)) /* Select Set Register */
#define TASK_GPIO_PXSELC(n)	(TASK_GPIO_BASE + (0x58 + (n)*0x100)) /* Select Clear Register */

#define TASK_GPIO_PXDIR(n)	(TASK_GPIO_BASE + (0x60 + (n)*0x100)) /* Direction Register */
#define TASK_GPIO_PXDIRS(n)	(TASK_GPIO_BASE + (0x64 + (n)*0x100)) /* Direction Set Register */
#define TASK_GPIO_PXDIRC(n)	(TASK_GPIO_BASE + (0x68 + (n)*0x100)) /* Direction Clear Register */

#define TASK_REG_GPIO_PXPIN(n)	REG32(TASK_GPIO_PXPIN((n)))

#define TASK_REG_GPIO_PXDAT(n)	REG32(TASK_GPIO_PXDAT((n)))
#define TASK_REG_GPIO_PXDATS(n)	REG32(TASK_GPIO_PXDATS((n)))
#define TASK_REG_GPIO_PXDATC(n)	REG32(TASK_GPIO_PXDATC((n)))

#define TASK_REG_GPIO_PXIM(n)	REG32(TASK_GPIO_PXIM((n)))
#define TASK_REG_GPIO_PXIMS(n)	REG32(TASK_GPIO_PXIMS((n)))
#define TASK_REG_GPIO_PXIMC(n)	REG32(TASK_GPIO_PXIMC((n)))

#define TASK_REG_GPIO_PXPE(n)	REG32(TASK_GPIO_PXPE((n)))
#define TASK_REG_GPIO_PXPES(n)	REG32(TASK_GPIO_PXPES((n)))
#define TASK_REG_GPIO_PXPEC(n)	REG32(TASK_GPIO_PXPEC((n)))

#define TASK_REG_GPIO_PXFUN(n)	REG32(TASK_GPIO_PXFUN((n)))
#define TASK_REG_GPIO_PXFUNS(n)	REG32(TASK_GPIO_PXFUNS((n)))
#define TASK_REG_GPIO_PXFUNC(n)	REG32(TASK_GPIO_PXFUNC((n)))

#define TASK_REG_GPIO_PXSEL(n)	REG32(TASK_GPIO_PXSEL((n)))
#define TASK_REG_GPIO_PXSELS(n)	REG32(TASK_GPIO_PXSELS((n)))
#define TASK_REG_GPIO_PXSELC(n)	REG32(TASK_GPIO_PXSELC((n)))

#define TASK_REG_GPIO_PXDIR(n)	REG32(TASK_GPIO_PXDIR((n)))
#define TASK_REG_GPIO_PXDIRS(n)	REG32(TASK_GPIO_PXDIRS((n)))
#define TASK_REG_GPIO_PXDIRC(n)	REG32(TASK_GPIO_PXDIRC((n)))

void task_gpio_init(uint8_t task);
void task_gpio_as_input(uint8_t pin);
void task_gpio_as_output(uint8_t pin);

void task_gpio_enable_pull(uint8_t pin);
void task_gpio_disable_pull(uint8_t pin);

int task_gpio_get_pin(uint8_t pin);
void task_gpio_set_pin(uint8_t pin);
void task_gpio_clear_pin(uint8_t pin);

void task_gpio_mask_irq(uint8_t pin);
void task_gpio_unmask_irq(uint8_t pin);

#endif /* __TASK_GPIO_H__ */
