/*
 * Keyboard scanning task.
 *
 * Copyright (C) 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "tasks.h"
#include "task_gpio.h"
#include "jzlcd.h"
#include "lcd.h"

extern vidinfo_t panel_info;

void plot_value(uint16_t xmin, uint16_t xmax, uint16_t ymin, uint16_t ymax, uint8_t value)
{
	uint16_t x, y;
	uint32_t pixel;

	for (y = ymin; (y < panel_info.jz_fb->h) && (y < ymax); y++)
	{
		pixel = (value >> (((y - ymin) * 8) / (ymax - ymin))) % 2 ? 0xffffffff : 0;

		for (x = xmin; (x < panel_info.jz_fb->w) && (x < xmax); x++)
		{
			set_pixel(x, y, pixel);
		}
	}
}

void init_keyscan()
{
	uint8_t pin;

	for (pin = 0; pin < GPIO_KEYIN_COUNT; pin++)
	{
		task_gpio_as_input(GPIO_KEYIN_ROW[pin]);
		task_gpio_enable_pull(GPIO_KEYIN_ROW[pin]);
	}

	for (pin = 0; pin < GPIO_KEYOUT_COUNT; pin++)
	{
		task_gpio_as_input(GPIO_KEYOUT_COL[pin]);
	}

	task_gpio_as_input(GPIO_POWER);
}

/* Tasks. */

void keyscan(uint8_t task)
{
	uint8_t column, row, value;

	init_keyscan();

	while (1)
	{
		/* Select each column and read the key states. */

		for (column = 0; column < GPIO_KEYOUT_COUNT; column++)
		{
			task_gpio_as_output(GPIO_KEYOUT_COL[column]);

			for (unsigned int delay = 0; delay < 1000; delay++)
				task_gpio_clear_pin(GPIO_KEYOUT_COL[column]);

			value = 0;

			for (row = 0; row < GPIO_KEYIN_COUNT; row++)
			{
				value = (value << 1) | (task_gpio_get_pin(GPIO_KEYIN_ROW[row]) ? 0 : 1);
			}

			/* Perform an operation indicating the status. */

			plot_value(column * 10, column * 10 + 10, 0, 10 * GPIO_KEYIN_COUNT, value);

			task_gpio_as_input(GPIO_KEYOUT_COL[column]);
		}

		plot_value(100, 110, 0, 80, task_gpio_get_pin(GPIO_POWER) ? 0xff : 0);
	}
}

void start_keyscan(uint8_t task)
{
	uint32_t args[] = {task};

	task_gpio_init(task);

	/* Invoke the task. */

	start_task(task, (void (*)()) keyscan, args, 1);
}
